=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2007-09-21 04:22:53 +0000
+++ bzrlib/dirstate.py	2007-09-28 07:09:05 +0000
@@ -212,6 +212,7 @@
 import zlib
 
 from bzrlib import (
+    debug,
     errors,
     inventory,
     lock,
@@ -340,6 +341,10 @@
         self._cutoff_time = None
         self._split_path_cache = {}
         self._bisect_page_size = DirState.BISECT_PAGE_SIZE
+        if 'hashcache' in debug.debug_flags:
+            self._sha1_file = self._sha1_file_and_mutter
+        else:
+            self._sha1_file = osutils.sha_file_by_name
 
     def __repr__(self):
         return "%s(%r)" % \
@@ -1128,7 +1133,7 @@
         # process this entry.
         link_or_sha1 = None
         if minikind == 'f':
-            link_or_sha1 = self._sha1_file(abspath, entry)
+            link_or_sha1 = self._sha1_file(abspath)
             executable = self._is_executable(stat_value.st_mode,
                                              saved_executable)
             if self._cutoff_time is None:
@@ -1182,13 +1187,11 @@
         """Return the os.lstat value for this path."""
         return os.lstat(abspath)
 
-    def _sha1_file(self, abspath, entry):
-        """Calculate the SHA1 of a file by reading the full text"""
-        f = file(abspath, 'rb', buffering=65000)
-        try:
-            return osutils.sha_file(f)
-        finally:
-            f.close()
+    def _sha1_file_and_mutter(self, abspath):
+        # when -Dhashcache is turned on, this is monkey-patched in to log
+        # file reads
+        trace.mutter("dirstate sha1 " + abspath)
+        return osutils.sha_file_by_name(abspath)
 
     def _is_executable(self, mode, old_executable):
         """Is this file executable?"""

=== modified file 'bzrlib/help_topics.py'
--- bzrlib/help_topics.py	2007-09-07 03:42:41 +0000
+++ bzrlib/help_topics.py	2007-09-28 07:09:05 +0000
@@ -265,6 +265,7 @@
                error.
 -Devil         Capture call sites that do expensive or badly-scaling
                operations.
+-Dhashcache    Log every time a working file is read to determine its hash.
 -Dhooks        Trace hook execution.
 -Dhpss         Trace smart protocol requests and responses.
 -Dindex        Trace major index operations.

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2007-09-24 20:13:29 +0000
+++ bzrlib/osutils.py	2007-09-28 07:09:05 +0000
@@ -590,6 +590,15 @@
     return s.hexdigest()
 
 
+def sha_file_by_name(fname):
+    """Calculate the SHA1 of a file by reading the full text"""
+    f = file(fname, 'rb', buffering=65000)
+    try:
+        return sha_file(f)
+    finally:
+        f.close()
+
+
 def sha_strings(strings, _factory=sha.new):
     """Return the sha-1 of concatenation of strings"""
     s = _factory()

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2007-07-21 18:27:19 +0000
+++ bzrlib/tests/test_dirstate.py	2007-09-28 07:09:05 +0000
@@ -1404,9 +1404,9 @@
         timestamp = super(InstrumentedDirState, self)._sha_cutoff_time()
         self._cutoff_time = timestamp + self._time_offset
 
-    def _sha1_file(self, abspath, entry):
+    def _sha1_file(self, abspath):
         self._log.append(('sha1', abspath))
-        return super(InstrumentedDirState, self)._sha1_file(abspath, entry)
+        return osutils.sha_file_by_name(abspath)
 
     def _read_link(self, abspath, old_link):
         self._log.append(('read_link', abspath, old_link))

